"""URLs for the ``media_library`` app."""
from django.conf.urls import patterns, url

from . import views


urlpatterns = patterns(
    '',
    url(r'^$', views.MediaLibraryEditView.as_view(),
        name='medialibrary_edit'),
    url(r'^upload/$', views.MediaLibraryUploadView.as_view(),
        name='medialibrary_upload'),
    url(r'^image/create/(?P<image_pk>\d+)/$',
        views.MediaItemImageCreateView.as_view(),
        name='mediaitem_image_create'),
    url(r'^image/update/(?P<image_pk>\d+)/$',
        views.MediaItemImageUpdateView.as_view(),
        name='mediaitem_image_update'),
    url(r'^video/create/$',
        views.MediaItemVideoCreateView.as_view(),
        name='mediaitem_video_create'),
    url(r'^video/update/(?P<pk>\d+)/$',
        views.MediaItemVideoUpdateView.as_view(),
        name='mediaitem_video_update'),
    url(r'^delete/(?P<pk>\d+)/$',
        views.MediaItemDeleteView.as_view(),
        name='mediaitem_delete'),
)



"""Forms for the ``media_library`` app."""
from django.utils.translation import get_language, ugettext_lazy as _

from hvad.forms import TranslatableModelForm
from multilingual_tags.forms.mixins import TaggingFormMixin

from . import models


class MediaItemImageForm(TaggingFormMixin, TranslatableModelForm):
    """Form to edit and create meta data for library images."""

    language = get_language()
    tag_field = {
        'name': 'tags',
        'label': _('Tags'),
        'help_text': _('You can add up to 3 tags separated by comma.'),
        'required': False,
        'max_tags': 3,
    }

    def __init__(self, user, image, *args, **kwargs):
        super(MediaItemImageForm, self).__init__(*args, **kwargs)
        self.image = image
        self.library = user.medialibraries.all()[0]
        self.fields['date'].widget.attrs.update({'data-class': 'datepicker'})

    def save(self, commit=True):
        self.instance.library = self.library
        self.instance.image = self.image
        return super(MediaItemImageForm, self).save(commit)

    class Meta:
        model = models.MediaItem
        fields = ['date', 'location']


class MediaItemVideoForm(TaggingFormMixin, TranslatableModelForm):

    language = get_language()
    tag_field = {
        'name': 'tags',
        'label': _('Tags'),
        'help_text': _('You can add up to 3 tags separated by comma.'),
        'required': False,
        'max_tags': 3,
    }

    def __init__(self, user, *args, **kwargs):
        super(MediaItemVideoForm, self).__init__(*args, **kwargs)
        self.library = user.medialibraries.all()[0]
        self.fields['date'].widget.attrs.update({'data-class': 'datepicker'})

    def save(self, commit=True):
        self.instance.library = self.library
        return super(MediaItemVideoForm, self).save(commit)

    class Meta:
        model = models.MediaItem
        fields = ['video', 'date', 'location']



"""Tests for the views of the ``media_library`` app."""
from django.test import TestCase
from django.core.urlresolvers import reverse

from django_libs.tests.factories import UserFactory
from django_libs.tests.mixins import ViewRequestFactoryTestMixin
from user_media.tests.factories import UserMediaImageFactory

from .. import models
from . import factories
from .. import views


# Test Mixins =================================================================

class MediaItemImageCRUDViewTestMixin(ViewRequestFactoryTestMixin):

    def get_view_kwargs(self):
        return {'image_pk': self.image.pk}

    def setUp(self):
        self.user = UserFactory()
        self.other_user = UserFactory()
        self.library = factories.MediaLibraryFactory(user=self.user)
        self.image = UserMediaImageFactory(
            user=self.user, content_object=self.library,
        )
        self.data = {
            'location': u'Berlin',
        }

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.is_callable(user=self.user)
        self.is_postable(
            user=self.user, data=self.data, to=reverse('medialibrary_edit'),
        )

        if hasattr(self, 'mediaitem'):
            self.is_not_callable(user=self.other_user)


class MediaItemVideoCRUDViewTestMixin(ViewRequestFactoryTestMixin):

    def setUp(self):
        self.user = UserFactory()
        self.other_user = UserFactory()
        self.library = factories.MediaLibraryFactory(user=self.user)
        self.data = {
            'location': u'Berlin',
            'video': u'https://youtube.com/watch?v=123456',
        }

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.is_callable(user=self.user)
        self.is_postable(
            user=self.user, data=self.data, to=reverse('medialibrary_edit'),
        )
        if hasattr(self, 'mediaitem'):
            self.is_not_callable(user=self.other_user)


# Tests =======================================================================

class MediaLibraryEditViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``MediaLibraryEditView`` view class."""
    view_class = views.MediaLibraryEditView

    def setUp(self):
        self.user = UserFactory()

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.is_callable(user=self.user)


class MediaItemDeleteViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``MediaItemDeleteView`` view class."""
    view_class = views.MediaItemDeleteView

    def get_view_kwargs(self):
        return {'pk': self.mediaitem.pk}

    def setUp(self):
        self.user = UserFactory()
        self.other_user = UserFactory()
        self.mediaitem = factories.MediaItemFactory(
            image=UserMediaImageFactory(),
            library__user=self.user)

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.is_callable(user=self.user)
        self.is_not_callable(user=self.other_user)
        self.is_postable(user=self.user, data={}, to=reverse(
            'medialibrary_edit'))
        self.assertEqual(models.MediaItem.objects.count(), 0, msg=(
            'The media item should be deleted.'
        ))


class MediaItemImageCreateViewTestCase(MediaItemImageCRUDViewTestMixin,
                                       TestCase):
    """Tests for the ``MediaItemImageCreateView`` view class."""
    view_class = views.MediaItemImageCreateView


class MediaItemImageUpdateViewTestCase(MediaItemImageCRUDViewTestMixin,
                                       TestCase):
    """Tests for the ``MediaItemImageUpdateView`` view class."""
    view_class = views.MediaItemImageUpdateView

    def setUp(self):
        super(MediaItemImageUpdateViewTestCase, self).setUp()
        self.mediaitem = factories.MediaItemFactory(
            image=self.image, library=self.library,
        )


class MediaItemVideoCreateViewTestCase(MediaItemVideoCRUDViewTestMixin,
                                       TestCase):
    """Tests for the ``MediaItemVideoCreateView`` view class."""
    view_class = views.MediaItemVideoCreateView


class MediaItemVideoUpdateViewTestCase(MediaItemVideoCRUDViewTestMixin,
                                       TestCase):
    """Tests for the ``MediaItemVideoUpdateView`` view class."""
    view_class = views.MediaItemVideoUpdateView

    def get_view_kwargs(self):
        return {'pk': self.mediaitem.pk}

    def setUp(self):
        super(MediaItemVideoUpdateViewTestCase, self).setUp()
        self.mediaitem = factories.MediaItemFactory(
            library=self.library, user=self.user,
        )


class MediaLibraryUploadViewTestCase(ViewRequestFactoryTestMixin, TestCase):
    """Tests for the ``MediaLibraryUploadView`` view class."""
    view_class = views.MediaLibraryUploadView

    def setUp(self):
        self.user = UserFactory()
        self.library = factories.MediaLibraryFactory(user=self.user)

    def test_view(self):
        self.should_redirect_to_login_when_anonymous()
        self.is_callable(user=self.user)
        self.assertEqual(models.MediaLibrary.objects.count(), 1, msg=(
            'There should be one media item automatically created.'
        ))



"""Admins for the models of the ``media_library`` app."""
from django.contrib import admin
from django.utils.translation import ugettext_lazy as _

from . import models


class MediaItemAdmin(admin.ModelAdmin):
    """Custom admin for the ``MediaItem`` model."""
    list_display = ['get_user', 'image', 'video']

    def get_user(self, obj):
        return obj.showreel.user
    get_user.short_description = _('user')


admin.site.register(models.MediaLibrary)
admin.site.register(models.MediaItem, MediaItemAdmin)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'MediaLibrary'
        db.create_table(u'media_library_medialibrary', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(related_name='medialibraries', to=orm['auth.User'])),
        ))
        db.send_create_signal(u'media_library', ['MediaLibrary'])

        # Adding model 'MediaItemTranslation'
        db.create_table(u'media_library_mediaitem_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('location', self.gf('django.db.models.fields.CharField')(max_length=256, blank=True)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['media_library.MediaItem'])),
        ))
        db.send_create_signal(u'media_library', ['MediaItemTranslation'])

        # Adding unique constraint on 'MediaItemTranslation', fields ['language_code', 'master']
        db.create_unique(u'media_library_mediaitem_translation', ['language_code', 'master_id'])

        # Adding model 'MediaItem'
        db.create_table(u'media_library_mediaitem', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('library', self.gf('django.db.models.fields.related.ForeignKey')(related_name='media_items', to=orm['media_library.MediaLibrary'])),
            ('image', self.gf('django.db.models.fields.related.ForeignKey')(blank=True, related_name='media_items', null=True, to=orm['user_media.UserMediaImage'])),
            ('video', self.gf('django.db.models.fields.URLField')(max_length=512, null=True, blank=True)),
            ('date', self.gf('django.db.models.fields.DateField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'media_library', ['MediaItem'])


    def backwards(self, orm):
        # Removing unique constraint on 'MediaItemTranslation', fields ['language_code', 'master']
        db.delete_unique(u'media_library_mediaitem_translation', ['language_code', 'master_id'])

        # Deleting model 'MediaLibrary'
        db.delete_table(u'media_library_medialibrary')

        # Deleting model 'MediaItemTranslation'
        db.delete_table(u'media_library_mediaitem_translation')

        # Deleting model 'MediaItem'
        db.delete_table(u'media_library_mediaitem')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'media_library.mediaitem': {
            'Meta': {'object_name': 'MediaItem'},
            'date': ('django.db.models.fields.DateField', [], {'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.related.ForeignKey', [], {'blank': 'True', 'related_name': "'media_items'", 'null': 'True', 'to': u"orm['user_media.UserMediaImage']"}),
            'library': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'media_items'", 'to': u"orm['media_library.MediaLibrary']"}),
            'video': ('django.db.models.fields.URLField', [], {'max_length': '512', 'null': 'True', 'blank': 'True'})
        },
        u'media_library.mediaitemtranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'MediaItemTranslation', 'db_table': "u'media_library_mediaitem_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'location': ('django.db.models.fields.CharField', [], {'max_length': '256', 'blank': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['media_library.MediaItem']"})
        },
        u'media_library.medialibrary': {
            'Meta': {'object_name': 'MediaLibrary'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'medialibraries'", 'to': u"orm['auth.User']"})
        },
        u'user_media.usermediaimage': {
            'Meta': {'object_name': 'UserMediaImage'},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100', 'null': 'True', 'blank': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_h': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_w': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_x': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_x2': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_y': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'thumb_y2': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['media_library']


"""Tests for the models of the ``media_library`` app."""
from django.test import TestCase

from user_media.models import UserMediaImage
from user_media.tests.factories import UserMediaImageFactory

from . import factories


class MediaLibraryTestCase(TestCase):
    """Tests for the ``MediaLibrary`` model class."""
    longMessage = True

    def test_instantiation(self):
        library = factories.MediaLibraryFactory()
        self.assertTrue(library.pk)


class MediaItemTestCase(TestCase):
    """Tests for the ``MediaItem`` model class."""
    longMessage = True

    def assertNotRaises(self, func, *args, **kwargs):
        try:
            func(*args, **kwargs)
        except Exception as ex:
            self.fail(msg=(
                '"{0}" should not have raised an exception, but raised'
                ' "{1}"'.format(repr(func), str(ex))
            ))

    def setUp(self):
        self.library = factories.MediaLibraryFactory()
        self.mediaitem = factories.MediaItemFactory(
            showreel=self.library,
            video='https://youtube.com/watch?v=123456',
        )
        self.umedia_image = UserMediaImageFactory()
        self.mediaitemimage = factories.MediaItemFactory(
            video=None, image=self.umedia_image,
        )

    def test_delete(self):
        self.mediaitemimage.delete()
        self.assertEqual(UserMediaImage.objects.count(), 0, msg=(
            'The user media images should have been deleted as well.'
        ))

    def test_instantiation(self):
        self.assertTrue(self.mediaitem.pk)

    def test_video_id(self):
        self.assertEqual(self.mediaitem.video_id, '123456', msg=(
            'The property should have returned the correct video id.'
        ))

    def test_clean(self):
        linklist = [
            'http://www.youtube.com/watch?v=-JyZLS2IhkQ',
            'https://www.youtube.com/watch?v=-JyZLS2IhkQ',
            'http://www.youtube.de/watch?v=-JyZLS2IhkQ',
            'https://youtube.com/watch?v=-JyZLS2IhkQ',
            ('https://www.youtube.com/watch?v=PguLNvCcOHQ'
             '&list=RDPguLNvCcOHQ#t=0'),
            'http://youtu.be/PguLNvCcOHQ?list=RDPguLNvCcOHQ ',
            'http://vimeo.com/channels/staffpicks/110140870',
            'http://vimeo.com/59777392',
            'http://vimeo.com/video/59777392',
            ('http://vimeo.com/groups/thedirectorofphotography/'
             'videos/110016243'),
        ]
        for link in linklist:
            self.mediaitem.video = link
            self.assertNotRaises(self.mediaitem.clean)
